<?php
/**
 * @class 		GJ_Fields
 * @package		GJ/Field/Classes
 * @author 		InwaveThemes
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class GJ_Fields {

	/** @var array Array of field login classes. */
	public $types;

	/**
	 * @var GJ_Fields The single instance of the class
	 */
	protected static $_instance = null;

	/**
	 * Main GJ_Fields Instance.
	 *
	 * Ensures only one instance of GJ_Fields is loaded or can be loaded.
	 *
	 * @static
	 * @return GJ_Fields
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Initialize types.
	 */
	public function __construct() {
		$this->init();
	}

	/**
	 * Load gateways and hook in functions.
	 */
	public function init() {
		$load_types = array(
			'GJ_Field_Text',
            'GJ_Field_Number',
            'GJ_Field_Price',
            'GJ_Field_Email',
			'GJ_Field_Url',
            'GJ_Field_Textarea',
			'GJ_Field_Checkbox',
			'GJ_Field_Checklist',
			'GJ_Field_Radios',
			'GJ_Field_SelectAdvanced',
			'GJ_Field_Taxonomy',
			'GJ_Field_Post',
			'GJ_Field_User',
			'GJ_Field_Datetime',
			'GJ_Field_Map',
			'GJ_Field_Image',
            'GJ_Field_CoverImage',
			'GJ_Field_Gallery',
			'GJ_Field_Editor',
			'GJ_Field_Social',
			'GJ_Field_CurriculumVitae',
			//'GJ_Field_Resume',
			'GJ_Field_Education',
			'GJ_Field_Experience',
			'GJ_Field_Award',
			'GJ_Field_Percent',
		);

		// Filter
		$load_types = apply_filters( 'gj_field_types', $load_types );

		// Get sort field option
		$ordering  = (array) get_option( 'gj_field_types_order' );
		$order_end = 999;

		// Load types in order
		foreach ( $load_types as $field ) {
			$load_field = is_string( $field ) ? new $field() : $field;

			if ( isset( $ordering[ $load_field->getType() ] ) && is_numeric( $ordering[ $load_field->getType() ] ) ) {
				// Add in position
				$this->types[ $ordering[ $load_field->type ] ] = $load_field;
			} else {
				// Add to end of the array
				$this->types[ $order_end ] = $load_field;
				$order_end++;
			}
		}
		ksort( $this->types );
	}

	/**
	 * Get types.
	 * @return GJ_Field_Abstract[]
	 */
	public function types() {
		$available_types = array();

		if ( sizeof( $this->types ) > 0 ) {
			foreach ( $this->types as $field ) {
				$available_types[ $field->getType() ] = $field;
			}
		}

		return $available_types;
	}

	/**
	 * Get type.
	 * @return GJ_Field_Abstract
	 */
	public function type($id = 'text') {
            $types = $this->types();
            foreach ( $types as $type ) {
                if($type->getType() == $id){
                    return clone $type;
                }
            }

          return clone $this->type('text');
	}

	public function getFieldSettings($post){
        $settings = new stdClass();
        $settings->active = $post->post_status == 'publish' ? 'true' : '';
        $settings->name = $post->post_title;
        $settings->id = $post->ID;
        $settings->group = $post->post_parent;
        $settings->type = $post->post_excerpt;
        $settings->metakey = get_post_meta($post->ID, 'metakey', true);
        $settings->order = $post->menu_order;
        $otherSettings = json_decode($post->post_content);
        if($otherSettings){
            $settings = (object) array_merge(
                (array) $settings, (array) $otherSettings);

        }
        return $settings;
    }

	/**
	 * Get available field.
	 *
	 * @return GJ_Field_Abstract
	 */

	public function getField($field, $postID = null) {
	    $type = $this->type($field->post_excerpt);
	    $settings = $this->getFieldSettings($field);
	    if($postID){
            $type->bindPostID($postID);
        }
        $type->bindSettings($settings);

		return $type;
	}

	/**
	 * Get available fields.
	 *
	 * @return array GJ_Field_Abstract
	 */

	public function getFields($post = null, $post_type = 'gj_job', $status = 'any') {
        $fields = [];
        $posts = get_posts(array(
            'post_type' => 'gj_fields',
            'posts_per_page' => -1,
            'post_status' => $status,
            'orderby' => 'menu_order',
            'order' => 'ASC',
            'meta_query' => array(
                array(
                    'key' => 'post_type',
                    'value' => $post_type,
                    'compare' => '=',
                )
            )
        ));

        foreach ($posts as $field){
            $fields[] = $this->getField($field, $post);
        }

        return $fields;
	}

	public function getGroupPosts($post_type){
        return get_posts(array(
            'post_type' => 'gj_field_groups',
            'posts_per_page' => -1,
            'post_status' => 'any',
            'orderby' => 'menu_order',
            'order' => 'ASC',
            'meta_query' => array(
                array(
                    'key' => 'post_type',
                    'value' => $post_type,
                    'compare' => '=',
                )
            )
        ));
    }

	public function getSchema($post_type = 'gj_job', $postID = null){
        $fields = $this->getFields($postID, $post_type );
        $post_type_config = isset(gj_field_manager_post_types()[$post_type]) ? gj_field_manager_post_types()[$post_type] : null;
        if($post_type_config['grouped']){
            $groups = $this->getGroupPosts($post_type);
            echo "var ".GJ_Field_Abstract::$JSSchemaVariableName." = {
            groups : [";
            do_action('gj_before_schema', $post_type, $postID);

            if($groups){
                $showIn = is_admin() ? 'backend' : 'dashboard';
                foreach ($groups as $group){
                    echo "{
                        legend : '{$group->post_title}',
                        settings : ".($group->post_content ? $group->post_content : '[]').",
                        fields : [";
                            do_action('gj_before_group_fields', $group, $post_type, $postID);
                            $inputs = array();
                            foreach ($fields as $field){
                                if($field->getSetting('showInput') === 'both' || $field->getSetting('showInput') === $showIn){
                                    if ($field->getSetting('group') == $group->ID) {
                                        $inputs[] = $field->input();
                                    }
                                }
                            }
                            echo implode(",", array_filter($inputs));
                            do_action('gj_after_group_fields', $group, $post_type, $postID);
                    echo "]},";
                }
            }
            do_action('gj_after_schema', $post_type, $postID);
            echo "]";
            echo "};";
        }else{
            echo "var ".GJ_Field_Abstract::$JSSchemaVariableName." = {
            fields : [";
                do_action('gj_before_schema', $post_type, $postID);
                $showIn = is_admin() ? 'backend' : 'dashboard';
                    $inputs = array();
                    foreach ($fields as $field){
                        if($field->getSetting('showInput') === 'both' || $field->getSetting('showInput') === $showIn){
                                $inputs[] = $field->input();
                        }
                    }
                    echo implode(",", array_filter($inputs));
                do_action('gj_after_schema', $post_type, $postID);
            echo "],";
            echo "};";
        }

    }

	public function getModels($post_type = 'gj_job', $postID = null){

        $fields = $this->getFields($postID, $post_type);

        $models = array();
        echo "var ".GJ_Field_Abstract::$JSModelVariableName." = {";

        if($fields){
            $showIn = is_admin() ? 'backend' : 'dashboard';
            foreach ($fields as $field){
                if($field->getSetting('showInput') === 'both' || $field->getSetting('showInput') === $showIn) {
                    $models[] = $field->model();
                }
            }
        }

        $models = apply_filters('gj_models', array_filter($models), $post_type, $postID);

        echo implode(",", $models);
        echo "};";

    }

    public static function getGroupSchema(){
	    $schema = array(
	        'fields' => array(
                array(
                    'label' => esc_html__( 'Title', 'gj' ),
                    'model'   => 'title',
                    'type' => 'input',
                    'inputType' => 'text',
                ),
                array(
                    'label' => esc_html__( 'Sub title', 'gj' ),
                    'model'   => 'sub_title',
                    'type' => 'input',
                    'inputType' => 'text',
                    'default' => '',
                ),
                array(
                    'label' => esc_html__( 'Custom class', 'gj' ),
                    'model'   => 'class',
                    'type' => 'input',
                    'inputType' => 'text',
                    'default' => '',
                ),
            )
        );

	    return apply_filters('gj_group_schema', $schema);
    }

    public function canEnqueueScript(){
	    $allow = false;
        if(is_admin()){
            global $current_screen;
            $allowScreens = array("edit-gj_state", "edit-gj_city");
            $pos_types = gj_field_manager_post_types();
            foreach ($pos_types as $pos_type){
                if(isset($pos_type['adminMetabox']) && $pos_type['adminMetabox']){
                    $allowScreens[] = $pos_type['post_type'];
                }
            }
            if(isset($current_screen) && in_array($current_screen->id, $allowScreens)){
                $allow = true;
            }
        }else{
            $pages = array();
            $pages[] = gj_get_page_id('dashboard');
            if(in_array(get_the_ID(), $pages)){
                $pos_types = gj_field_manager_post_types();
                foreach ($pos_types as $pos_type){
                    if(isset($pos_type['enqueueScriptsInDashboardEnpoint']) && $pos_type['enqueueScriptsInDashboardEnpoint']){
                        $endpoints[] = $pos_type['enqueueScriptsInDashboardEnpoint'];
                    }
                }
                $currentEndPoint = gj_current_dashboard_endpoint();
                if(in_array($currentEndPoint, $endpoints)){
                    $allow = true;
                }
            }
        }

        return apply_filters('gj_can_enqueue_field_scripts', $allow);
    }

	public function registerScripts(){
        $js_version = GJ_VERSION;

        $js_footer = true;
        if(is_admin()){
            $js_footer = false;
        }
        wp_register_script('vue', GJ_PLUGIN_URL.'assets/vue/vue.min.js', array(), $js_version, $js_footer);
        wp_register_script('sortable', GJ_PLUGIN_URL.'assets/vue/Sortable.min.js', array(), $js_version, $js_footer); //@1.8.4
        wp_register_script('vuedraggable', GJ_PLUGIN_URL.'assets/vue/vuedraggable.umd.min.js', array(), $js_version, $js_footer); //@2.9.12
        wp_register_script('vue-multiselect', GJ_PLUGIN_URL.'assets/vue/vue-multiselect.min.js', array('vue'), $js_version, $js_footer);
        wp_register_style('vue-multiselect', GJ_PLUGIN_URL.'assets/vue/vue-multiselect.min.css');
        wp_register_style( 'datetimepicker', GJ_PLUGIN_URL . 'assets/css/jquery.datetimepicker.css', array() );
        wp_register_script( 'datetimepicker', GJ_PLUGIN_URL . 'assets/js/jquery.datetimepicker.full.min.js', array( 'jquery' ), $js_version, $js_footer );
        wp_register_script('vfg', GJ_PLUGIN_URL.'assets/vue/vfg.js', array('vue'), $js_version, $js_footer);

        wp_localize_script('vfg',   'vfgi18n', array(
            'fieldIsRequired' => esc_html__("This field is required!", 'gj'),
            'invalidFormat' => esc_html__("Invalid format!", 'gj'),
            'numberTooSmall' => esc_html__("The number is too small! Minimum: {0}", 'gj'),
            'numberTooBig' => esc_html__("The number is too big! Maximum: {0}", 'gj'),
            'invalidNumber' => esc_html__("Invalid number", 'gj'),
            'invalidInteger' => esc_html__("The value is not an integer", 'gj'),
            'textTooSmall' => esc_html__("The length of text is too small! Current: {0}, Minimum: {1}", 'gj'),
            'textTooBig' => esc_html__("The length of text is too big! Current: {0}, Maximum: {1}", 'gj'),
            'thisNotText' => esc_html__("This is not a text!", 'gj'),
            'thisNotArray' => esc_html__("This is not an array!", 'gj'),
            'selectMinItems' => esc_html__("Select minimum {0} items!", 'gj'),
            'selectMaxItems' => esc_html__("Select maximum {0} items!", 'gj'),
            'invalidDate' => esc_html__("Invalid date!", 'gj'),
            'dateIsEarly' => esc_html__("The date is too early! Current: {0}, Minimum: {1}", 'gj'),
            'invalidEmail' => esc_html__("Invalid e-mail address!", 'gj'),
            'invalidURL' => esc_html__("Invalid URL!", 'gj'),
            'invalidCard' => esc_html__("Invalid card format!", 'gj'),
            'invalidCardNumber' => esc_html__("Invalid card number!", 'gj'),
            'invalidTextContainNumber' => esc_html__("Invalid text! Cannot contains numbers or special characters", 'gj'),
            'invalidTextContainSpec' => esc_html__("Invalid text! Cannot contains special characters", 'gj'),
            'noOptions' => esc_html__('No List Found','gj'),
            'fetchingOptions' => esc_html__('Fetching data...', 'gj'),
            'noResult' => esc_html__("No elements found. Consider changing the search query", 'gj'),
            'maxElements' => esc_html__("Maximum of {{ max }} options selected. First remove a selected option to select another.", 'gj'),
            'countryLabel' => esc_html__("Country", 'gj'),
            'cityLabel' => esc_html__("City", 'gj'),
            'stateLabel' => esc_html__("State", 'gj'),
            'maxUploadFileSize' => esc_html__("Maximum upload file size is", 'gj'),
            'uploadNewFile' => esc_html__("Upload new file", 'gj'),
            'selectFile' => esc_html__("Select file", 'gj'),
        ));
            
        wp_register_style('vfg', GJ_PLUGIN_URL.'assets/vue/vfg.css');
        wp_register_script('fields-manager', GJ_PLUGIN_URL.'assets/vue/fields-manager.js', array('jquery', 'vue'), $js_version, true);
        wp_localize_script('fields-manager', 'fieldmanageri18n', array(
            'fieldRequired' => esc_html__("This field is required!", 'gj'),
            'specialNotAccept' => esc_html__("Spaces and special characters are not accepted", 'gj'),
            'metakeyExists' => esc_html__("This Metakey has been used by an other field", 'gj'),
            'fieldTypes' => esc_html__("Field types", 'gj'),
            'fieldsList' => esc_html__("Fields list", 'gj'),
            'drangElement' => esc_html__("Drang and drop element to the group area", 'gj'),
            'confirmDeleteField' => esc_html__("Are you sure delete this field ?", 'gj'),
        ));
        wp_register_style('fields-manager', GJ_PLUGIN_URL.'assets/vue/fields-manager.css');
    }

	public function enqueueTypeScripts( $types = array()){
        if($types){
            foreach ($types as $type){
                $type = $this->type($type);
                if($type){
                    $type->enqueue();
                }
            }
        }
    }

	public function enqueueScripts( $force = false){

        if($force || $this->canEnqueueScript() || (!is_admin() && is_single() && get_post_type() == 'gj_job')){
            wp_enqueue_script('vue');

            if(!$force){
                $params = self::getParams();
                $fields = $this->getFields($params[1], $params[0]);
                foreach ($fields as $field){
                    $field->enqueue();
                }
            }

            if(is_admin()){
                wp_enqueue_script('vue-multiselect');
                wp_enqueue_style('vue-multiselect');
                wp_enqueue_style( 'datetimepicker');
                wp_enqueue_script( 'datetimepicker');
            }

            do_action('gj_fields_enqueue');

            wp_enqueue_script('sortable');
            wp_enqueue_script('vuedraggable');
            wp_enqueue_script('vfg');
            wp_enqueue_style('vfg');
        }
    }

    static function AdminJSForm($postType, $postID){
        $fields = GJ_Fields::instance();
        if($fields){
            ?>
            <script type="text/javascript">
                <?php $fields->getSchema($postType, $postID); ?>
                <?php $fields->getModels($postType, $postID); ?>
                var vmPostApp;
                jQuery(document).ready(function(){
                    vmPostApp = new Vue({
                        el: "#admin-post-app",
                        components: {
                            "vue-form-generator": VueFormGenerator.component,
                        },
                        methods: {
                            prettyJSON: function(json) {
                                if (json) {
                                    json = JSON.stringify(json, undefined, 4);
                                    json = json.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
                                    return json.replace(/("(\\u[a-zA-Z0-9]{4}|\\[^u]|[^\\"])*"(\s*:)?|\b(true|false|null)\b|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)/g, function (match) {
                                        var cls = "number";
                                        if (/^"/.test(match)) {
                                            if (/:$/.test(match)) {
                                                cls = "key";
                                            } else {
                                                cls = "string";
                                            }
                                        } else if (/true|false/.test(match)) {
                                            cls = "boolean";
                                        } else if (/null/.test(match)) {
                                            cls = "null";
                                        }
                                        return "<span class=\"" + cls + "\">" + match + "</span>";
                                    });
                                }
                            },
                            validate(isValid, errors, self){
                            }
                        },
                        data: {
                            model: <?php echo GJ_Field_Abstract::$JSModelVariableName; ?>,
                            schema:  <?php echo GJ_Field_Abstract::$JSSchemaVariableName; ?>,
                            formOptions: {
                                validateAfterLoad: false,
                                validateAfterChanged: true
                            }
                        }
                    });
                    jQuery(document).on('click', '.editor-post-publish-button', function (e) {
                       e.preventDefault();
                    });
                    var savingCustomFields = false;
                    if(wp.data && typeof wp.data.subscribe === 'function'){
                        setTimeout(function() {
                            jQuery(document).on('click', '.editor-post-publish-button, .editor-post-save-draft', function (e) {
                                e.preventDefault();
                                savingCustomFields = false;
                                if(!vmPostApp.$children[0].validate(false, true)){
                                    var metabox = jQuery('#<?php echo $postType; ?>-box');
                                    if(metabox.hasClass('closed')){
                                        metabox.find('h2').trigger('click');
                                    }
                                    return false;
                                }else{
                                    wp.data.subscribe(function () {
                                        var success = wp.data.select('core/editor').didPostSaveRequestSucceed();
                                        var isAutosavingPost = wp.data.select('core/editor').isAutosavingPost();
                                        if(success && !isAutosavingPost && !savingCustomFields){
                                            savingCustomFields = true;
                                            var data = Object.assign({}, vmPostApp.model);
                                            data.action = 'gj_save_post';
                                            data.postID = jQuery('#post_ID').val();

                                            var formData = new FormData();
                                            GJAppendFormdata(formData, data);

                                            jQuery.ajax({
                                                type: "POST",
                                                url: gj.ajax_url,
                                                data : formData,
                                                dataType: "json",
                                                cache: false,
                                                contentType: false,
                                                processData: false,
                                                //contentType: "application/json; charset=UTF-8",
                                                complete: function(data){
                                                    //savingCustomFields = false;
                                                },
                                                error : function () {
                                                    savingCustomFields = false;
                                                }
                                            });
                                        }

                                    });
                                }
                            });
                        }, 500)
                    }else{
                        jQuery("form#post").submit(function(e) {
                            var form = jQuery(this);
                            if(vmPostApp.$children[0].validate(false, true)){
                                var data = Object.assign({}, vmPostApp.model);
                                data.action = 'gj_save_post';
                                data.postID = jQuery('#post_ID').val();

                                var formData = new FormData();
                                GJAppendFormdata(formData, data);
                                form.find('input[type="hidden"].extra').each(function(){
                                    data[jQuery(this).attr('name')] = jQuery(this).val();
                                });
                                jQuery.ajax({
                                    type: "POST",
                                    url: gj.ajax_url,
                                    data : formData,
                                    dataType: "json",
                                    cache: false,
                                    contentType: false,
                                    processData: false,
                                    //contentType: "application/json; charset=UTF-8",
                                    complete: function(data){
                                    },
                                    error : function () {
                                    }
                                });
                            }
                        })
                    }
                });
            </script>
            <?php
        }
    }

    static function DashboardJSForm($postType, $postID){
        $fields = GJ_Fields::instance();
        if($fields){
            ?>
            <script type="text/javascript">
                var vmPostApp;
                jQuery(document).ready(function () {
                    <?php $fields->getSchema($postType, $postID); ?>
                    <?php $fields->getModels($postType, $postID); ?>
                    vmPostApp = new Vue({
                        el: "#dashboard-post-app",
                        components: {
                            "vue-form-generator": VueFormGenerator.component,
                        },
                        data: {
                            model: <?php echo GJ_Field_Abstract::$JSModelVariableName; ?>,
                            schema:  <?php echo GJ_Field_Abstract::$JSSchemaVariableName; ?>,
                            formOptions: {
                                validateAfterLoad: false,
                                validateAfterChanged: true
                            }
                        }
                    });
                });
            </script>
            <?php
        }
    }

    static function ApplicationJSForm($postType, $postID){
        $fields = GJ_Fields::instance();
        if($fields){
            ?>
            <script type="text/javascript">
                var vmApplicationApp;
                jQuery(document).ready(function () {
                    <?php $fields->getSchema('gj_application', 0); ?>
                    <?php $fields->getModels('gj_application', 0); ?>

                    vmApplicationApp = new Vue({
                        el: "#application-app",
                        components: {
                            "vue-form-generator": VueFormGenerator.component,
                        },
                        data: {
                            model: <?php echo GJ_Field_Abstract::$JSModelVariableName; ?>,
                            schema:  <?php echo GJ_Field_Abstract::$JSSchemaVariableName; ?>,
                            formOptions: {
                                validateAfterLoad: false,
                                validateAfterChanged: true
                            }
                        }
                    });
                });
            </script>
            <?php
        }
    }

    function savePost(){
	    //save core fields

        $postID = intval($_POST['postID']);
        do_action('gj_before_save_post', $postID);
        $postType = get_post_type($postID);
        $fields = $this->getFields($postID, $postType);
        if($fields){
            foreach ($fields as $field){
                if(is_admin() && ($field->getSetting('showInput') === 'both' || $field->getSetting('showInput') === 'backend')){
                    $field->save();
                }elseif(!is_admin() && ($field->getSetting('showInput') === 'both' || $field->getSetting('showInput') === 'dashboard')){
                    $field->save();
                }
            }
        }

        do_action('gj_after_save_post', $postID);
    }

    static function getParams(){
	    if(is_admin()){
            global $current_screen;
            $postType = isset($current_screen->id) ? $current_screen->id : "";
            $postID = isset($_GET['post']) ? intval($_GET['post']) : "";
            return [$postType, $postID ];
        }else{
	        if(get_the_ID() == gj_get_page_id('dashboard')){
                $endPoint = gj_current_dashboard_endpoint();
                switch ($endPoint){
                    case 'company':
                        $user = GJ_User::get_user();
                        return ["gj_employer", $user->get_employer_id()];
                    case 'resume':
                        $user = GJ_User::get_user();
                        return ["gj_candidate", $user->get_candidate_id()];
                    default :
                        $post_types = gj_field_manager_post_types();
                        foreach ($post_types as $post_type){
                            if(isset($post_type['enqueueScriptsInDashboardEnpoint']) && $post_type['enqueueScriptsInDashboardEnpoint'] == $endPoint){
                                global $wp_query;
                                $postID = isset($wp_query->query_vars['postID']) ? $wp_query->query_vars['postID'] : 0;
                                return [$post_type['post_type'], $postID];
                            }
                        }

                        return ["", ""];
                }
            }else if(is_single() && get_post_type() == 'gj_job'){
                return ["gj_application", 0];
            }
        }

        return ["", ""];
    }

    static function application_footer_scripts(){
        if(is_single() && get_post_type() == 'gj_job'){
            $params = self::getParams();
            self::ApplicationJSForm($params[0], $params[1]);
        }
    }

    static function print_footer_scripts(){
        $self = new GJ_Fields();
        if($self->canEnqueueScript()){
            $params = self::getParams();
            self::DashboardJSForm($params[0], $params[1]);
        }
    }

    static function admin_print_footer_scripts(){
        $self = new GJ_Fields();
        if($self->canEnqueueScript()){
            $params = self::getParams();
            self::AdminJSForm($params[0], $params[1]);
        }
    }

    static function add_metabox() {
        $post_types = gj_field_manager_post_types();
        foreach ($post_types as $post_type){
            if(isset($post_type['adminMetabox']) && $post_type['adminMetabox']){
                add_meta_box($post_type['post_type'].'-box', esc_html__('Fields Metabox', 'gj'), array(__CLASS__, 'metabox_html'), $post_type['post_type'], 'normal', 'high');
            }
        }
    }

    static function metabox_html() {
        ?>
        <div class="gj-metabox wp-clearfix">
            <div class="container" id="admin-post-app">
                <vue-form-generator :schema="schema" :model="model" :options="formOptions"></vue-form-generator>
            </div>
        </div>
        <?php
    }
}

//GJ_Fields::instance();